
from datetime import date
from numpy import number

import base64
# support functions for encoding data

def getUint16(bytes_array, offset : number, littleEndian : bool) :
    if(littleEndian):
        return (bytes_array[offset+1]*256 + bytes_array[offset])
    return (bytes_array[offset]*256 + bytes_array[offset+1])

def getInt16(bytes_array, offset : number, littleEndian : bool) : 
    value = getUint16(bytes_array, offset, littleEndian)
    if (value & 0x8000) :
        value -= 0x10000
    return value
  
def getUint32(bytes_array, offset : number, littleEndian : bool) :
    if(littleEndian):
        return (bytes_array[offset+3]*256*256*256 + bytes_array[offset+2]*256*256 + bytes_array[offset+1]*256 + bytes_array[offset])
    return (bytes_array[offset+0]*256*256*256 + bytes_array[offset+1]*256*256 + bytes_array[offset+2]*256 + bytes_array[offset+3])

def byteToHexString(byte : bytes) : 
    return hex(byte)[2:].zfill(2)

def oldParseData(data):
    data = base64.b64decode(data)

    if (len(data) == 20) :
        o = {
            'd0_len_cat_0': getUint16(data, 0, False),
            'd0_len_cat_1': getUint16(data, 2, False),
            'd0_len_cat_2': getUint16(data, 4, False),
            'd1_len_cat_0': getUint16(data, 6, False),
            'd1_len_cat_1': getUint16(data, 8, False),
            'd1_len_cat_2': getUint16(data, 10, False),
            'len_cat_3': getUint16(data, 12, False),
            'overspeed_cnt': getUint16(data, 14, False),
            'avg_speed': data[16],
        }
        # add summed values
        o['d0'] = o['d0_len_cat_0'] + o['d0_len_cat_1'] + o['d0_len_cat_2'] + o['len_cat_3']
        o['d1'] = o['d1_len_cat_0'] + o['d1_len_cat_1'] + o['d1_len_cat_2']

        if (data[17] == 0) :
            o['total_counter'] = getUint16(data, 18, False)
        if (data[17] == 1) :
            o['ref1'] = getInt16(data, 18, False)
        if (data[17] == 2) :
            o['ref2'] = getInt16(data, 18, False)
        if (data[17] == 3) :
            o['m1'] = getInt16(data, 18, False)
        if (data[17] == 4) :
            o['m2'] = getInt16(data, 18, False)
        if (data[17] == 5) :
            batsup_type = (data[18] >> 4) & 0x0F
            batval = getUint16(data, 18, False) & 0x0FFF
            if (batsup_type == 0) :
                o['bat1'] = batval * (2.0*2.2)/(1.2*4096.0)
            if (batsup_type == 1) :
                o['bat2'] = batval * (2.0*2.2)/(1.2*4096.0)
            if (batsup_type == 2) :
                o['bat3'] = batval * (2.0*2.2)/(1.2*4096.0)
            if (batsup_type == 3) :
                o['bat4'] = batval * (2.0*2.2)/(1.2*4096.0)
            if (batsup_type == 4) :
                o['rfsup'] = batval * (2.0*2.2)/(1.2*4096.0)
            if (batsup_type == 5) :
                o['ti_temp'] = batval
            if (batsup_type == 6) :
                o['vsup'] = batval * (2.000*2)/4096.0 
        return o

    #####
    # LENGTH 30
    #####
    if (len(data) == 30) :
        o = {'part':1}
        #12bit counts from 3x4 table 
        v = getUint32(data, 0, True)
        o['len0_speed0_cnt12'] = (v & 0xFFF)
        o['len0_speed1_cnt12'] = ((v >> 12) & 0xFFF)
        v = getUint32(data, 3, True)
        o['len0_speed2_cnt12'] = (v & 0xFFF)
        o['len0_speed3_cnt12'] = ((v >> 12) & 0xFFF)    
        v = getUint32(data, 6, True)
        o['len1_speed0_cnt12'] = (v & 0xFFF)
        o['len1_speed1_cnt12'] = ((v >> 12) & 0xFFF)
        v = getUint32(data, 9, True)
        o['len1_speed2_cnt12'] = (v & 0xFFF)
        o['len1_speed3_cnt12'] = ((v >> 12) & 0xFFF)    
        v = getUint32(data, 12, True)
        o['len2_speed0_cnt12'] = (v & 0xFFF)
        o['len2_speed1_cnt12'] = ((v >> 12) & 0xFFF)
        v = getUint32(data, 15, True)
        o['len2_speed2_cnt12'] = (v & 0xFFF)
        o['len2_speed3_cnt12'] = ((v >> 12) & 0xFFF)
        
        #full-range 32bit counts from 3x4 table
        v = getUint16(data, 22, True)
        o['len3_cnt12'] = ((v >> 4) & 0xFFF) 
        r32_type = v & 0xF
        v = getUint32(data, 18, True)
        if (r32_type == 0) :
            o['len0_speed0_cnt32'] = v
        if (r32_type == 1) :
            o['len0_speed1_cnt32'] = v
        if (r32_type == 2) :
            o['len0_speed2_cnt32'] = v
        if (r32_type == 3) :
            o['len0_speed3_cnt32'] = v
        if (r32_type == 4) :
            o['len1_speed0_cnt32'] = v
        if (r32_type == 5) :
            o['len1_speed1_cnt32'] = v
        if (r32_type == 6) :
            o['len1_speed2_cnt32'] = v
        if (r32_type == 7) :
            o['len1_speed3_cnt32'] = v
        if (r32_type == 8) :
            o['len2_speed0_cnt32'] = v
        if (r32_type == 9) :
            o['len2_speed1_cnt32'] = v
        if (r32_type == 10) :
            o['len2_speed2_cnt32'] = v
        if (r32_type == 11) :
            o['len2_speed3_cnt32'] = v
        if (r32_type == 12) :
            o['len3_cnt32'] = v
        if (r32_type == 13) :
            o['systime32'] = v
        
        #top speeds + barometric pressure
        v = getUint32(data, 24, True)
        o['pressure'] = getUint16(data, 28, True) | ((v & 0x03) << 16)
        o['topspeed_len0_n'] = ((v >> 27) & 0x1F) 
        o['topspeed_len0_p'] = ((v >> 22) & 0x1F) 
        o['topspeed_len1_n'] = ((v >> 17) & 0x1F) 
        o['topspeed_len1_p'] = ((v >> 12) & 0x1F) 
        o['topspeed_len2_n'] = ((v >> 7) & 0x1F) 
        o['topspeed_len2_p'] = ((v >> 2) & 0x1F) 
        return o

    #####
    # length = 26 OR 28
    #####
    if (len(data) == 26 or len(data) == 28) :
        o = {
            'part': 2,
            'temperature': (data[3] - 65.0),
            'relative_humidity': (data[2] >> 1) & 0x7F,
        }
        if (len(data) == 28) :
            o['ds18b20'] = getInt16(data, 26, True) * 0.0625
        
        # byte to hex function
        o['cfgmem'] =  byteToHexString(data[21]) + "_" + byteToHexString(data[22]) + byteToHexString(data[23]) + byteToHexString(data[24]) + byteToHexString(data[25])
        
        r16_type = data[4] & 0x07

        if (r16_type == 0) :
            o['total_counter'] = getUint16(data, 0, False)
        if (r16_type == 1) :
            o['ref1'] = getInt16(data, 0, False)
        if (r16_type == 2) :
            o['ref2'] = getInt16(data, 0, False)
        if (r16_type == 3) :
            o['m1'] = getInt16(data, 0, False)
        if (r16_type == 4) :
            o['m2'] = getInt16(data, 0, False)
        if (r16_type == 5) :
            batsup_type = (data[0] >> 4) & 0x0F
            batval = getUint16(data, 0, False) & 0x0FFF
            if (batsup_type == 0) :
                o['bat1'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 1) :
                o['bat2'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 2) :
                o['bat3'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 3) :
                o['bat4'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 4) :
                o['rfsup'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 5) :
                o['ti_temp'] = batval
            
            if (batsup_type == 6) :
                o['vsup'] = batval * (2.000*2)/4096.0 # 1/2 VCC
        
        tp_size = (data[4] >> 4) & 0x0F
        if(tp_size > 10):
            tp_size = 10

        tp_list = []
        
        for i in range(0, tp_size) :
            v = getUint32(data,5+3*(i>>1), True) >> (12 * (1 - (i & 1)))
            tp_list.append({'timediff' : ((v & 0x7F) << 2), 'speed_cat' : ((v >> 7) & 0x07), 'len_cat' : ((v >> 10) & 0x03)})
        
        o['trespass'] = tp_list  
        
        return o

    #####
    # length = 7
    #####
    if (len(data) == 7) :
        o = {
        'cal0': getInt16(data, 0, True),
        'cal1': getInt16(data, 2, True),
        'cal_slow_mode_z': getInt16(data, 4, True),
        'cal_success': data[6],
        }
        return o

    #####
    # length = 5
    #####
    if (len(data) == 5) :
        o = {
            'TX_ACK_REPORT': (data[0]==1 and data[1]==2 and data[2]==3 and data[3]==4 and data[4]==5),
        }
        return o

    return {}

# main parsing function
# 
def parseData(data):
    data = base64.b64decode(data)
    #####
    # LENGTH 28
    #####
    if (len(data) == 28) :
        o = {'part':1}
        #12bit counts from 3x4 table 
        v = getUint32(data, 0, True)
        o['len0_speed0_cnt12'] = (v & 0xFFF)
        o['len0_speed1_cnt12'] = ((v >> 12) & 0xFFF)
        v = getUint32(data, 3, True)
        o['len0_speed2_cnt12'] = (v & 0xFFF)
        o['len0_speed3_cnt12'] = ((v >> 12) & 0xFFF)    
        v = getUint32(data, 6, True)
        o['len1_speed0_cnt12'] = (v & 0xFFF)
        o['len1_speed1_cnt12'] = ((v >> 12) & 0xFFF)
        v = getUint32(data, 9, True)
        o['len1_speed2_cnt12'] = (v & 0xFFF)
        o['len1_speed3_cnt12'] = ((v >> 12) & 0xFFF)    
        v = getUint32(data, 12, True)
        o['len2_speed0_cnt12'] = (v & 0xFFF)
        o['len2_speed1_cnt12'] = ((v >> 12) & 0xFFF)
        v = getUint32(data, 15, True)
        o['len2_speed2_cnt12'] = (v & 0xFFF)
        o['len2_speed3_cnt12'] = ((v >> 12) & 0xFFF)
        
        o['ds18b20'] = getInt16(data, 26, True) * 0.0625

        r16_type = (data[27] >> 4) & 0x0F
        u = getUint16(data, 24, False)  #uint16, big endian, high8=msg[24], low8=msg[25]
        v = getInt16(data, 24, False)  #int16, big endian, high8=msg[24], low8=msg[25]
        if (r16_type == 0):
            o['total_counter'] = u
        if (r16_type == 1):
            o['ref1'] = v
        if (r16_type == 2):
            o['ref2'] = v
        if (r16_type == 3):
            o['m1'] = v
        if (r16_type == 4):
            o['m2'] = v
        if (r16_type == 5):
            batsup_type = (u >> 12) & 0x0F
            batval = u & 0x0FFF
            if (batsup_type == 0):
                o['bat1'] = batval * 11.0/12288.0
            if (batsup_type == 1):
                o['bat2'] = batval * 11.0/12288.0
            if (batsup_type == 2):
                o['bat3'] = batval * 11.0/12288.0
            if (batsup_type == 3):
                o['bat4'] = batval * 11.0/12288.0
            if (batsup_type == 4):
                o['rfsup'] = batval * 11.0/12288.0
            if (batsup_type == 5):
                o['ti_temp'] = batval
            if (batsup_type == 6):
                o['vsup'] = batval * 12.0/12288.0
            # if (r16_type == 6):
            #     #RH+t

        return o

    #####
    # length = 26 OR 28
    #####
    if (len(data) == 26) :
        o = {
            'part': 2,
            'temperature': (data[3] - 65.0),
            'relative_humidity': (data[2] >> 1) & 0x7F,
        }
        
        # byte to hex function
        o['cfgmem'] =  byteToHexString(data[21]) + "_" + byteToHexString(data[22]) + byteToHexString(data[23]) + byteToHexString(data[24]) + byteToHexString(data[25])
        
        r16_type = data[4] & 0x07

        if (r16_type == 0) :
            o['total_counter'] = getUint16(data, 0, False)
        if (r16_type == 1) :
            o['ref1'] = getInt16(data, 0, False)
        if (r16_type == 2) :
            o['ref2'] = getInt16(data, 0, False)
        if (r16_type == 3) :
            o['m1'] = getInt16(data, 0, False)
        if (r16_type == 4) :
            o['m2'] = getInt16(data, 0, False)
        if (r16_type == 5) :
            batsup_type = (data[0] >> 4) & 0x0F
            batval = getUint16(data, 0, False) & 0x0FFF
            if (batsup_type == 0) :
                o['bat1'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 1) :
                o['bat2'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 2) :
                o['bat3'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 3) :
                o['bat4'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 4) :
                o['rfsup'] = batval * (2.0*2.2)/(1.2*4096.0)
            
            if (batsup_type == 5) :
                o['ti_temp'] = batval
            
            if (batsup_type == 6) :
                o['vsup'] = batval * (2.000*2)/4096.0 # 1/2 VCC
        
        tp_size = (data[4] >> 4) & 0x0F
        if(tp_size > 10):
            tp_size = 10

        tp_list = []
        
        for i in range(0, tp_size) :
            v = getUint32(data,5+3*(i>>1), True) >> (12 * (1 - (i & 1)))
            tp_list.append({'timediff' : ((v & 0x7F) << 2), 'speed_cat' : ((v >> 7) & 0x07), 'len_cat' : ((v >> 10) & 0x03)})
        
        o['trespass'] = tp_list  
        
        return o

    #####
    # length = 7
    #####
    if (len(data) == 7) :
        o = {
        'cal0': getInt16(data, 0, True),
        'cal1': getInt16(data, 2, True),
        'cal_slow_mode_z': getInt16(data, 4, True),
        'cal_success': data[6],
        }
        return o

    #####
    # length = 5
    #####
    if (len(data) == 5) :
        o = {
            'TX_ACK_REPORT': (data[0]==1 and data[1]==2 and data[2]==3 and data[3]==4 and data[4]==5),
        }
        return o
  
    return {}


def week_number_of_month(day, month, year):
    return date(year, month, day).isocalendar()[1]
